function [x, w] = gaussJacobi(alpha, N, a, b)
import Measure.*
% function [x, w] = gaussJacobi(alpha, N, a, b)
%
% This function computes the integration points and weights for integration
% with the Gauss-Jacobi method (Gauss integration with Jacobi polynomials
% with weight function w(x)=(1-x)^alpha, with alpha = 1,2) in the interval
% [a, b] and N integration points.
%
% input:
% alpha - exponent for weight function
% N - number of integration points, size 1x1
% a - left limit of the integration interval, size 1x1
% b - right limit of the integration interval, size 1x1
%
% output:
% x - integration points, size Nx1
% w - integration weights, size Nx1

% author:
% Lena Meyer
% Fachbereich Mathematik und Statistik
% Universitaet Konstanz
% Lena.Meyer@uni-konstanz.de
%
% date: 12/04/2013

% subroutines:
% coefficientsJacobi - to compute the coefficients beta and gamma for the
%                       Gauss-Jacobi method

% used routines: error, intervalTransformation

% variables: beta, gamma - coefficients for the Jacobi polynomials
%            sqrtGamma - square root of gamma added with a zero
%            J - matrix to compute the roots of the polynomials
%            ev, ew - eigenvectors and eigenvalues of J
%            tmpX, tmpW - integration points and weights for the interval
%                         [-1,1]
%            fac - factor for interval transformation


% check for necessary values for the number of integration points and the
% integration interval:
% check if N > 0, and set it to default value 1 if not
if N < 1
    error('FEM:WrongDegree', ['You choose N=%1.0f as the degree of ', ...
        'the polynomial. This is not possible.'], N);
end

% check the interval, if it has wrong orientation switch it
if a > b
    error('FEM:WrongInterval', ['WARNING: You choose the interval ', ...
        '[%1.0f,%1.0f], this is not valid.'], a, b);
end


% Jacobi polynomials: p_n+1 = (x-beta_n)*p_n - gamma_n^2*p_n-1

% get the coefficients for gamma and beta:
[beta, gamma] = coefficientsJacobi(alpha, N);

% set right gamma for the matrix, zero is necessary for spdiags
sqrtGamma = [sqrt(gamma); 0];

% create the matrix for the polynomials
J = spdiags([sqrtGamma(2:end), beta, sqrtGamma(1:end-1)], [-1,0,1], N, N);


% compute the eigenvalues and eigenvectors of J
% (this are the integration points and the integration weights)
[ev, ew] = eig(full(J));
tmpX = diag(ew);
tmpW = gamma(1)*(ev(1,:)).^2; % in gamma(1) is <p0,p0>_w stored


% check coefficient for the weight function and set factor for interval
% transformation
switch alpha
    case 1
        fac = 1/2;
    case 2
        fac = 1/4;
    otherwise
        error('You choose alpha wrong, alpha has to be 1 or 2');
end


% transformation to the right interval
[x, w] = intervalTransformation(tmpX, tmpW', -1, 1, a, b, fac);
end % function (gaussJacobi)



function [beta, gamma] = coefficientsJacobi(alpha, N)
% function [beta, gamma] = coefficientsJacobi(alpha, N)
%
% This function is to compute the coefficients used for Jacobian Matrix to
% compute the Jacobi integration points and weights for w(x) = (1-x)^alpha
%
% input:
% alpha - exponent for weight functions
% N - number of integration points and weights
%
% output:
% beta - beta coefficients for the matrix
% gamma - gamma coefficients for the matrix with gamma_0 = int_-1^1 w(x) dx

% subroutines: none
% used routines: none
% variables: k - iteration index for recursion formula


% create the necessary vectors:
beta = zeros(N,1);
gamma = zeros(N,1);

% check coefficient for weight function and set beta and gamma with the
% given formulas
switch alpha
    case 1 % weight function w(x) = 1 - x
        % set the first entry (special cases):
        beta(1) = -1/3;
        gamma(1) = 2;
        
        % set second entry (special case for gamma):
        if N > 1
            beta(2) = -1/15;
            gamma(2) = 2/9;
            
            % general recursion formula:
            for k = 2:1:N-1
                beta(k+1) = -1/((2*k + 1)*(2*k + 3));
                gamma(k+1) = k*(k+1)/(2*k +1)^2;
            end % for (k)
        end % if (N)
        
    case 2 % weight function w(x) = (1-x)^2
        % set first entry (special case)
        beta(1) = -1/2;
        gamma(1) = 8/3;
        
        % set second entry (special case for gamma):
        if N > 1
            beta(2) = -1/6;
            gamma(2) = 3/20;
            
            % general recursion formula:
            for k = 2:1:N-1
                beta(k+1) = -1/((k+1)*(k+2));
                gamma(k+1) = (k^2*(k + 2)^2)/((k+1)^2*(2*k+1)*(2*k + 3));
            end % for (k)
        end % if (N)
        
    otherwise % case not implemented
        error('You choose a wrong alpha, it has to be 1 or 2');
end % switch (alpha)
end % function (coefficientsJacobi)
