function [x, w] = gaussLegendre(N, a, b)
import Measure.*
% function [x, w] = gaussLegendre(N, a, b)
%
% This function computes the integration points and integration weights for
% integration with the Gauss-Legendre method (Gauss integration with
% Legendre polynomials and weight function w(x) = 1) in the interval [a, b]
% and N integration points.
%
% input:
% N - number of integration points, size 1x1
% a - left limit of the integration interval, size 1x1
% b - right limit of the integration interval, size 1x1
%
% output:
% x - integration points, size Nx1
% w - integration weights, size Nx1

% author:
% Lena Meyer
% Fachbereich Mathematik und Statistik
% Universitaet Konstanz
% Lena.Meyer@uni-konstanz.de
%
% date: 12/04/2013

% subroutines:
% coefficientsLegendre - to get the coefficients for the Legendre
%                        polynomials

% used routines: error, intervalTransformation

% variables: beta, gamma - coefficients of the Legendre polynomial
%            sqrtGamma - square root of gamma added with zero
%            J - matrix for the polynomials
%            ev, ew - eigenvectors and eigenvalues of J
%            tmpX, tmpW - integration points and weights in interval [-1,1]


% check for necessary values for N and the interval:
% check if N > 0, if not set N to default value
if N < 1
    error('FEM:WrongDegree', ['You choose N=%1.0f as the degree of ', ...
        'the polynomial.'], N);
end

% check the interval, if it is not valid, we switch start and end point
if a > b
    error('FEM:WrongInterval', ['You choose the interval ', ...
        '[%1.0f,%1.0f], this is not valid.'], a, b);
end


% special case N = 1
if N == 1
    % set point and weight for one integration point
    tmpX = 0;
    tmpW = 2;
else
    % Legendre polynomials: p_n+1 = x*p_n - gamma_n^2*p_n-1
    % get the coefficient matrix for beta and gamma
    [beta, gamma] = coefficientsLegendre(N);
    
    % set right gamma, with zero for spdiags
    sqrtGamma = [sqrt(gamma); 0];
    
    % create the matrix for the polynomials
    J = spdiags([sqrtGamma(2:end), beta, sqrtGamma(1:end-1)], ...
        [-1, 0, 1], N, N);
    
    % compute the eigenvalues and eigenvectors of L (that are the
    % integration points and the integration weights)
    [ev, ew] = eig(full(J));
    tmpX = diag(ew);
    tmpW = gamma(1)*(ev(1,:)).^2; %in gamma(1) is <p0,p0>_w stored
end % if (special case)


% compute the right points with linear transformation
[x,w] = intervalTransformation(tmpX, tmpW', -1, 1, a, b); 

end % function (gaussLegendre)



function [beta, gamma] = coefficientsLegendre(N)
% function [beta, gamma] = coefficientsLegendre(N)
%
% This function is to compute the coefficients used for Jacobian Matrix to
% compute the Legendre integration points and weights.
%
% input:
% N - number of integration points and weights
%
% output:
% beta - beta coefficients for the matrix
% gamma - gamma coefficients for the matrix with gamma_0 = int_-1^1 w(x) dx

% subroutines: none
% used routines: none
% variables: k - iteration index for recursion formula


% create the necessary vectors:
beta = zeros(N,1);
gamma = zeros(N,1);

% set the first entry (special cases)
gamma(1) = 2;

% set second entry (special case)
if N > 1
    gamma(2) = 1/3;
    
    % general recursion formula:
    for k = 2:1:N-1
        gamma(k+1) = 1/(4 - k^(-2));
    end % for (k)
end % if (N)

end % function (coefficientsLegendre)
 
