

classdef funcSys < handle
    properties
        nfunc;
        domain;
        range;
        funcs; 
    end
    methods
        function fs = funcSys(nfunc,varargin)
        % only the number of functions is required in FEM solution space;
        % The concrete functions are required only for the Basis on the refercence cell.
           if ~isa(nfunc,'numeric')
               error('The required argument gives the total number in the function system');
           end
           fs.nfunc = nfunc;
           if ~isempty(varargin)
               g = varargin{1};
               if nfunc == length(g)
                 if funcSys.check_funcsys(g)
                   fs.domain = g{1}.domain;
                   fs.range = g{1}.range; 
                   fs.funcs = g;
                 end
               else
                   error('Not enough functions in the function system');
               end
           end 
        end
        %
        function s = at(fs,x)
            % the k-th row is values of the k-th function at x.
            if ~isa(fs,'funcSys')  
                error('Here is to evaluate a function systerm at coordinate x');
            end
            if sum(size(x)==fs.domain.dim)<1
                error('The coordinate of x is not properly given');
            end
            if size(x,2)==fs.domain.dim
                x = x';
            end
            if ~check_xInDomain(fs.domain,x)
                error('x is not in the domain of function system');
            end
            s = zeros(fs.nfunc, size(x,2));
            for k = 1:fs.nfunc
             s(k,:) = fs.funcs{k}.at(x);
            end
        end
        %
        function s = derivate(fs,m)
            % return the derivative in k-th direction of m-th function in
            % the function system
            if ~isa(fs,'funcSys')
                error('One of the argument is a function system');
            end
            if ~isa(m,'numeric') || ~isequal(m, abs(floor(m))) || length(m)~=2
                error('m=[m1,m2] indicate to return the derivative in m2-th direction of m1-th function in function system');
            end
            s = func(fs.domain, fs.range, fs.d{m(1),m(2)});
        end
        
        % check independence
        function orn = check_indep(fs, varargin) 
            if isempty(varargin)
                xmin = min(fs.domain.nodes);
                xmax = max(fs.domain.nodes);
                x=rand(fs.nfunc,fs.domain.dim)*diag(xmax)+repmat(xmin, fs.nfunc, 1); 
            else
                x=varargin{1};
            end 
            M = fs.at(x); 
            orn =  abs(det(M))>1e-6; 
        end
        % display 
        function plot_funcSys(fs)
            if fs.domain.dim==1
               figure 
               hold on
                xmin = min(fs.domain.nodes);
                xmax = max(fs.domain.nodes);
                nx = 100;
                x=rand(nx,fs.domain.dim)*diag(xmax-xmin)+repmat(xmin, nx, 1);
                y = fs.at(x);
                plot(x', y','.') 
               hold off
            else
                display('there is no display for systems in 2D and 3D');
            end
        end
    end
    %
  
    methods (Static, Access=private)
        function orn = check_funcsys(g)  
            orn = 1; 
            for k=1:length(g)
                orn = orn && isa(g{k},'func');
                orn = orn && (g{k}.domain == g{1}.domain);
                orn = orn && (g{k}.range == g{1}.range);
            end
            if ~orn
                error('The functions are either not function or given on different domain or range');
            end 
        end
            
    end %methods
end