% A triangulation is a set of polytopes.
% A triangulation in 1D is a set of 0-polytopes (the reference cell is point) and 1-polytopes
% (the reference cell is segment). The representation of a poytope includes
% a reference cell and an affine mapping.
% point and segment are speciall triangulations. 

classdef triangulation1D < triangulation
    properties (GetAccess = public, SetAccess = protected)
        nodes;          % the nodes in one triangulation, nnode x 1
        cell2node;      % node index of cells, ncell x dim
        ref = {};       % all the reference type in one triangulation         
        poly2node = {};  % node index of polytopes, npoly x k (k=0 or 1 in 1D)
        poly2cell = {}; % all cells that contain the polytope
        poly2dim = [];  % the dimension of a polytope, k for k-polytope
        poly2ref = [];  % the reference polytope of a polytope
        poly2aff = {};  % the affine mapping of a polytope to its reference (falsch!!)
        node2poly = {}; % the set of polytopes that contain node k 
        cell2poly = []; % the polytope number that the cell corresponds to
        cell2ref =[];   % the reference polytope of cells
        cell2Aff = {};  % the affine mapping from cell to the reference
        npoly;          % the total number of polytopes in one triangulation
        ncell;          % the total number of cells in one triangulation
        nnode;          % the total number of nodes in one trianguation
        nref;           % the totoal number of reference types.
        mdim;            % maximal dimension of the polytopes in one triangulation
        %dim;           % embeded dimensiont (the problem dimension)
        loc2glob = [];  % the table of the local index in cells and the global index of nodes.  
    end
    methods 
        function tr = triangulation1D(varargin)
            tr@triangulation(1);
            % the possible arguments are orderly nodes, cells and reference cell.
            % 0 argument  : tr is a empty triangulation
            % 2 arguments : tr is reference cells, either point or segment
            % 3 arguments : tr is a triangulation, its each polytope has
            %               a reference cell and an affine mapping
            if length(varargin) == 2
                tr.nodes = varargin{1};
                tr.cell2node = varargin{2};
                tr.dim = 1;  % this is a finite element problem in R^1.
                if size(tr.nodes,2)~=tr.dim && size(tr.nodes,1)~=tr.dim 
                    error('The node points in 1D triangulation is wrongly given');
                end
                if size(tr.nodes,1)==tr.dim
                    tr.nodes = tr.nodes';
                end
                if size(tr.cell2node,2)> tr.dim+1 || size(tr.cell2node,1)~=1
                    error('The reference cell in 1D triangulation is a row vector 1x(k+1) for k-th polytope,k=0,1');
                end  
            end
            if length(varargin) == 3 
                tr.nodes = varargin{1};
                tr.cell2node = varargin{2}; 
                tr.dim = 1;
                Tref = varargin{3};
                if size(tr.nodes,2)~=tr.dim && size(tr.nodes,1)~=tr.dim 
                    error('The node points in 1D triangulation is wrongly given');
                end
                if size(tr.nodes,1)==tr.dim
                    tr.nodes = tr.nodes';
                end
                if size(tr.cell2node,2)~=size(Tref.cell2node,2) 
                    error('N cells are N row vectors, each row contains the node index in one cell');
                end  
                %
                tr.nnode = size(tr.nodes,1);
                tr.ncell = size(tr.cell2node, 1);
                
                % initialization
                tr.node2poly = cell(tr.nnode,1);
                tr.cell2poly = zeros(tr.ncell,1);
                tr.loc2glob = zeros(tr.ncell, Tref.nnode);
                %
                tr.nref = Tref.nref + 1; 
                if Tref.nref>0
                 tr.ref = Tref.ref;
                end
                tr.ref{tr.nref}=Tref;
                tr.mdim = Tref.mdim;
                tr.cell2ref = tr.nref;
                
                nply = 0;
                for c= 1:tr.ncell  % collect all the polytopes cell by cell
                   K = tr.cell2node(c,:);  % node index of a cell
                   tr.loc2glob(c,:) = K; 
                   [A,b]=get_Aff(tr,c);
                   tr.cell2Aff{c} = {A,b}; 
                   for p = 1:Tref.npoly % the polytopes in one cell
                      p2n = K(Tref.poly2node{p});  % node index of a polytope in this cell
                      % check if this polytope is already in the triangulation 
                      isin = 0;
                      for n = 1:nply
                       if isequal(p2n, tr.poly2node{n})
                           isin = n; 
                       end
                      end
                      if isin  % if already in
                          tr.poly2cell{isin} = [c, tr.poly2cell{isin}];
                      else     % if not in
                          nply = nply+1;
                          tr.poly2node{nply} = p2n;
                          tr.poly2dim(nply) = Tref.poly2dim(p); 
                          tr.poly2cell{nply} = c;
                          %tr.poly2aff{nply} = 
                          if isequal(p2n, K) 
                              tr.cell2poly(c) = nply;  
                              tr.poly2ref(nply) = tr.nref;
                          else
                              tr.poly2ref(nply) = Tref.poly2ref(p);
                          end 
                          tr.poly2aff{nply} = tr.ref{tr.poly2ref(nply)}.poly2aff*tr.nodes(p2n,:);
                          for nd = 1:length(p2n)
                              if isempty(tr.node2poly{p2n(nd)})
                                tr.node2poly{p2n(nd)} = [nply, nd];
                              else
                                tr.node2poly{p2n(nd)} = [[nply, nd]; tr.node2poly{p2n(nd)}]; 
                              end
                          end
                      end
                   end %  polytopes in one cell
                end %  cells
                tr.npoly = nply; 
            end 
        end % end of the constructor
        %
        %
        function val = get_node2cell(tr, k)
            poly = tr.node2poly{k}; 
            cellIn = find(tr.poly2ref(poly(:,1))==tr.cell2ref); 
            val = zeros(length(cellIn),2);
            for i = 1:length(cellIn)
             val(i,:)= [tr.poly2cell{poly(cellIn(i),1)}, poly(cellIn(i),2)];
            end
        end
      %
      function [A,b] = get_Aff(tr,c)
          % x=s*A+b, x = Nxdim
          % b is a row vector
         cell = tr.cell2node(c,:); 
         p = tr.nodes(cell,:); 
         d = size(cell, 2);
         b = p(1,:);
         A =  p(2:d,:)-repmat(p(1,:),d-1,1); 
      end
      function s = get_xInRef(tr, c, x)
         [A,b] = get_Aff(tr,c);
         if size(x,2)~=size(b,2)
           x=x';
         end
         if size(x,2)~=size(b,2)
            error('the given point is not in this cell');
         end
         b = x-repmat(b, size(x,1),1); 
         s = b/A;
      end
      function x = get_sInCell(tr, c, s) 
         [A,b] = get_Aff(tr,c);
         if size(s,2)~=size(b,2)
           s=s';
         end
         if size(s,2)~=size(b,2)
            error('the given point is not in this Reference cell');
         end  
         x = s*A + repmat(b, size(s,1),1);
      end 
    end
end